@@ -32,6 +32,7 @@
 #include <libxml/xpathInternals.h>
 #include <libxml/parserInternals.h>
 #include <libxml/uri.h>
+#include <libxml/xpointer.h>
 #include "xslt.h"
 #include "xsltInternals.h"
 #include "xsltutils.h"
@@ -97,6 +98,114 @@ xsltXPathFunctionLookup (xmlXPathContextPtr ctxt,
  *									*
  ************************************************************************/
 
+static void
+xsltDocumentFunctionLoadDocument(xmlXPathParserContextPtr ctxt, xmlChar* URI)
+{
+    xsltTransformContextPtr tctxt;
+    xmlURIPtr uri;
+    xmlChar *fragment;
+    xsltDocumentPtr xsltdoc;
+    xmlDocPtr doc;
+    xmlXPathContextPtr xptrctxt = NULL;
+    xmlXPathObjectPtr object = NULL;
+
+    tctxt = xsltXPathGetTransformContext(ctxt);
+    if (tctxt == NULL) {
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+			   "document() : internal error tctxt == NULL\n");
+	valuePush(ctxt, xmlXPathNewNodeSet(NULL));
+	return;
+    } 
+	
+    uri = xmlParseURI((const char *) URI);
+    if (uri == NULL) {
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+			   "document() : failed to parse URI\n");
+	valuePush(ctxt, xmlXPathNewNodeSet(NULL));
+	return;
+    } 
+    
+    /*
+     * check for and remove fragment identifier
+     */
+    fragment = (xmlChar *)uri->fragment;
+    if (fragment != NULL) {
+	uri->fragment = NULL;
+	URI = xmlSaveUri(uri);
+	xsltdoc = xsltLoadDocument(tctxt, URI);
+	xmlFree(URI);
+    } else
+	xsltdoc = xsltLoadDocument(tctxt, URI);
+    xmlFreeURI(uri);
+    
+    if (xsltdoc == NULL) {
+	if ((URI == NULL) ||
+	    (URI[0] == '#') ||
+	    (xmlStrEqual(tctxt->style->doc->URL, URI))) {
+	    doc = tctxt->style->doc;
+	} else {
+	    valuePush(ctxt, xmlXPathNewNodeSet(NULL));
+
+	    if (fragment != NULL)
+		xmlFree(fragment);
+
+	    return;
+	}
+    } else
+	doc = xsltdoc->doc;
+
+    if ( fragment == NULL ) {
+	valuePush(ctxt,
+		  xmlXPathNewNodeSet((xmlNodePtr) doc));
+	return;
+    }
+	
+    /* use XPointer of HTML location for fragment ID */
+#ifdef LIBXML_XPTR_ENABLED
+    xptrctxt = xmlXPtrNewContext(doc, NULL, NULL);
+    if (xptrctxt == NULL) {
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+			   "document() : internal error xptrctxt == NULL\n");
+	goto out_fragment;
+    }
+
+    object = xmlXPtrEval(fragment, xptrctxt);
+#endif
+    xmlFree(fragment);
+	if (xptrctxt != NULL)
+    		xmlXPathFreeContext(xptrctxt);
+
+    if (object == NULL)
+	goto out_fragment;
+	
+    switch (object->type) {
+    case XPATH_NODESET:
+	break;
+    case XPATH_UNDEFINED:
+    case XPATH_BOOLEAN:
+    case XPATH_NUMBER:
+    case XPATH_STRING:
+    case XPATH_POINT:
+    case XPATH_USERS:
+    case XPATH_XSLT_TREE:
+    case XPATH_RANGE:
+    case XPATH_LOCATIONSET:
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+			   "document() : XPointer does not select a node set: #%s\n", 
+			   fragment);
+	goto out_object;
+    }
+    
+    valuePush(ctxt, object);
+    return;
+
+out_object:
+    xmlXPathFreeObject(object);
+
+out_fragment:
+    valuePush(ctxt, xmlXPathNewNodeSet(NULL));
+}
+
 /**
  * xsltDocumentFunction:
  * @ctxt:  the XPath Parser context
@@ -108,7 +217,6 @@ xsltXPathFunctionLookup (xmlXPathContextPtr ctxt,
 void
 xsltDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs)
 {
-    xsltDocumentPtr doc;
     xmlXPathObjectPtr obj, obj2 = NULL;
     xmlChar *base = NULL, *URI;
 
@@ -193,7 +301,8 @@ xsltDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs)
             xmlNodePtr target;
 
             target = obj2->nodesetval->nodeTab[0];
-            if (target->type == XML_ATTRIBUTE_NODE) {
+            if ((target->type == XML_ATTRIBUTE_NODE) ||
+	        (target->type == XML_PI_NODE)) {
                 target = ((xmlAttrPtr) target)->parent;
             }
             base = xmlNodeGetBase(target->doc, target);
@@ -215,32 +324,9 @@ xsltDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs)
         if (URI == NULL) {
             valuePush(ctxt, xmlXPathNewNodeSet(NULL));
         } else {
-            xsltTransformContextPtr tctxt;
-
-            tctxt = xsltXPathGetTransformContext(ctxt);
-            if (tctxt == NULL) {
-                xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
-                                 "document() : internal error tctxt == NULL\n");
-                valuePush(ctxt, xmlXPathNewNodeSet(NULL));
-            } else {
-                doc = xsltLoadDocument(tctxt, URI);
-                if (doc == NULL) {
-		    if ((xmlStrEqual(URI, BAD_CAST "")) ||
-			(xmlStrEqual(tctxt->style->doc->URL, URI))) {
-			valuePush(ctxt, xmlXPathNewNodeSet(
-				    (xmlNodePtr)tctxt->style->doc));
-		    } else {
-			valuePush(ctxt, xmlXPathNewNodeSet(NULL));
-		    }
-		} else {
-                    /* TODO: use XPointer of HTML location for fragment ID */
-                    /* pbm #xxx can lead to location sets, not nodesets :-) */
-                    valuePush(ctxt,
-                              xmlXPathNewNodeSet((xmlNodePtr) doc->doc));
-                }
-            }
-            xmlFree(URI);
-        }
+	    xsltDocumentFunctionLoadDocument( ctxt, URI );
+	    xmlFree(URI);
+	}
     }
     xmlXPathFreeObject(obj);
     if (obj2 != NULL)
@@ -262,7 +348,9 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
     xmlChar *key = NULL, *value;
     const xmlChar *keyURI;
     xsltTransformContextPtr tctxt;
-
+    xsltDocumentPtr oldDocumentPtr;
+    xmlDocPtr oldXPathDocPtr;
+    
     if (nargs != 2) {
 	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"key() : expects two arguments\n");
@@ -346,8 +434,37 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
 	value = obj2->stringval;
 
 	tctxt = xsltXPathGetTransformContext(ctxt);
-
+	oldDocumentPtr = tctxt->document;
+	oldXPathDocPtr = tctxt->xpathCtxt->doc;
+	if ((ctxt->context->doc != NULL) &&
+		    (tctxt->document->doc != ctxt->context->doc)) {
+	    /*
+	     * The xpath context document needs to be changed.  If the
+	     * current context document is a node-set, we must use an
+	     * xsltDocument associated with the node-set, which may or
+	     * may not currently exist.
+	     */
+	    if (xmlStrEqual((const xmlChar *)ctxt->context->doc->name,
+	    		BAD_CAST " fake node libxslt")) {	/* node-set */
+		/*
+		 * Check whether we already have an xsltDocument set up
+		 */
+		if (ctxt->context->doc->_private == NULL)	/* nope */
+		    ctxt->context->doc->_private =
+		    	xsltNewDocument(tctxt, ctxt->context->doc);
+	        tctxt->document = ctxt->context->doc->_private;
+	    }
+	    else {
+	        tctxt->document = xsltFindDocument(tctxt, ctxt->context->doc);
+	        if (tctxt->document == NULL)
+	            tctxt->document = oldDocumentPtr;
+	        else
+	            tctxt->xpathCtxt->doc = ctxt->context->doc;
+	    }
+	}
 	nodelist = xsltGetKey(tctxt, key, keyURI, value);
+	tctxt->document = oldDocumentPtr;
+	tctxt->xpathCtxt->doc = oldXPathDocPtr;
 	valuePush(ctxt, xmlXPathWrapNodeSet(
 		        xmlXPathNodeSetMerge(NULL, nodelist)));
     }
@@ -436,6 +553,11 @@ xsltFormatNumberFunction(xmlXPathParserContextPtr ctxt, int nargs)
 	CAST_TO_STRING;
 	decimalObj = valuePop(ctxt);
 	formatValues = xsltDecimalFormatGetByName(sheet, decimalObj->stringval);
+	if (formatValues == NULL) {
+	    xsltTransformError(tctxt, NULL, NULL,
+		    "format-number() : undeclared decimal format '%s'\n", 
+		    decimalObj->stringval);
+	}
 	/* Intentional fall-through */
     case 2:
 	CAST_TO_STRING;
@@ -447,14 +569,16 @@ xsltFormatNumberFunction(xmlXPathParserContextPtr ctxt, int nargs)
 	XP_ERROR(XPATH_INVALID_ARITY);
     }
 
-    if (xsltFormatNumberConversion(formatValues,
-				   formatObj->stringval,
-				   numberObj->floatval,
-				   &result) == XPATH_EXPRESSION_OK) {
-	valuePush(ctxt, xmlXPathNewString(result));
-	xmlFree(result);
+    if (formatValues != NULL) {
+	if (xsltFormatNumberConversion(formatValues,
+				       formatObj->stringval,
+				       numberObj->floatval,
+				       &result) == XPATH_EXPRESSION_OK) {
+	    valuePush(ctxt, xmlXPathNewString(result));
+	    xmlFree(result);
+	}
     }
-    
+
     xmlXPathFreeObject(numberObj);
     xmlXPathFreeObject(formatObj);
     xmlXPathFreeObject(decimalObj);
