@@ -64,8 +64,10 @@ void
 xsltNamespaceAlias(xsltStylesheetPtr style, xmlNodePtr node) {
     xmlChar *sprefix;
     xmlNsPtr sNs;
+    const xmlChar *shref;
     xmlChar *rprefix;
     xmlNsPtr rNs;
+    const xmlChar *rhref;
 
     sprefix = xsltGetNsProp(node, (const xmlChar *)"stylesheet-prefix",
 	                   XSLT_NAMESPACE);
@@ -81,37 +83,68 @@ xsltNamespaceAlias(xsltStylesheetPtr style, xmlNodePtr node) {
 	    "namespace-alias: result-prefix attribute missing\n");
 	goto error;
     }
+    
     if (xmlStrEqual(sprefix, (const xmlChar *)"#default")) {
+        /*
+	 * Do we have a default namespace previously declared?
+	 */
 	sNs = xmlSearchNs(node->doc, node, NULL);
+	if (sNs == NULL)
+	    shref = NULL;	/* No - set NULL */
+	else
+	    shref = sNs->href;	/* Yes - set for nsAlias table */
     } else {
 	sNs = xmlSearchNs(node->doc, node, sprefix);
+ 
+	if ((sNs == NULL) || (sNs->href == NULL)) {
+	    xsltTransformError(NULL, style, node,
+	        "namespace-alias: prefix %s not bound to any namespace\n",
+					sprefix);
+	    goto error;
+	} else
+	    shref = sNs->href;
     }
-    if ((sNs == NULL) || (sNs->href == NULL)) {
-	xsltTransformError(NULL, style, node,
-	    "namespace-alias: prefix %s not bound to any namespace\n",
-	                 sprefix);
-	goto error;
-    }
+
+    /*
+     * When "#default" is used for result, if a default namespace has not
+     * been explicitly declared the special value UNDEFINED_DEFAULT_NS is
+     * put into the nsAliases table
+     */
     if (xmlStrEqual(rprefix, (const xmlChar *)"#default")) {
 	rNs = xmlSearchNs(node->doc, node, NULL);
+	if (rNs == NULL)
+	    rhref = UNDEFINED_DEFAULT_NS;
+	else
+	    rhref = rNs->href;
     } else {
 	rNs = xmlSearchNs(node->doc, node, rprefix);
+
+        if ((rNs == NULL) || (rNs->href == NULL)) {
+	    xsltTransformError(NULL, style, node,
+	        "namespace-alias: prefix %s not bound to any namespace\n",
+					rprefix);
+	    goto error;
+	} else
+	    rhref = rNs->href;
     }
-    if ((rNs == NULL) || (rNs->href == NULL)) {
-	xsltTransformError(NULL, style, node,
-	    "namespace-alias: prefix %s not bound to any namespace\n",
-	                 rprefix);
-	goto error;
+    /*
+     * Special case if #default is used for stylesheet and no default has
+     * been explicitly declared.  We use style->defaultAlias for this
+     */
+    if (shref == NULL) {
+        if (rNs != NULL)
+            style->defaultAlias = rNs->href;
+    } else {
+        if (style->nsAliases == NULL)
+	    style->nsAliases = xmlHashCreate(10);
+        if (style->nsAliases == NULL) {
+	    xsltTransformError(NULL, style, node,
+	        "namespace-alias: cannot create hash table\n");
+	    goto error;
+        }
+        xmlHashAddEntry((xmlHashTablePtr) style->nsAliases,
+	            shref, (void *) rhref);
     }
-    if (style->nsAliases == NULL)
-	style->nsAliases = xmlHashCreate(10);
-    if (style->nsAliases == NULL) {
-	xsltTransformError(NULL, style, node,
-	    "namespace-alias: cannot create hash table\n");
-	goto error;
-    }
-    xmlHashAddEntry((xmlHashTablePtr) style->nsAliases,
-	            sNs->href, (void *) rNs->href);
 
 error:
     if (sprefix != NULL)
@@ -247,6 +280,15 @@ xsltGetPlainNamespace(xsltTransformContextPtr ctxt, xmlNodePtr cur,
 	style = xsltNextImport(style);
     }
 
+    if (URI == UNDEFINED_DEFAULT_NS) {
+        xmlNsPtr dflt;
+	dflt = xmlSearchNs(cur->doc, cur, NULL);
+        if (dflt == NULL)
+	    return NULL;
+	else
+	    URI = dflt->href;
+    }
+
     if (URI == NULL)
 	URI = ns->href;
 
@@ -368,26 +410,45 @@ xsltGetNamespace(xsltTransformContextPtr ctxt, xmlNodePtr cur, xmlNsPtr ns,
 	style = xsltNextImport(style);
     }
 
-    if (URI == NULL)
+    if (URI == UNDEFINED_DEFAULT_NS) {
+        xmlNsPtr dflt;
+	dflt = xmlSearchNs(cur->doc, cur, NULL);
+	if (dflt != NULL)
+	    URI = dflt->href;
+	else
+	    return NULL;
+    } else if (URI == NULL)
 	URI = ns->href;
 
+    /*
+     * If the parent is an XML_ELEMENT_NODE, and has the "equivalent"
+     * namespace as ns (either both default, or both with a prefix
+     * with the same href) then return the parent's ns record
+     */
     if ((out->parent != NULL) &&
 	(out->parent->type == XML_ELEMENT_NODE) &&
 	(out->parent->ns != NULL) &&
+	(((out->parent->ns->prefix == NULL) && (ns->prefix == NULL)) ||
+	 ((out->parent->ns->prefix != NULL) && (ns->prefix != NULL))) &&
 	(xmlStrEqual(out->parent->ns->href, URI)))
 	ret = out->parent->ns;
     else {
-	if (ns->prefix != NULL) {
-	    ret = xmlSearchNs(out->doc, out, ns->prefix);
-	    if ((ret == NULL) || (!xmlStrEqual(ret->href, URI))) {
-		ret = xmlSearchNsByHref(out->doc, out, URI);
-	    }
-	} else {
+        /*
+	 * do a standard namespace search for ns in the output doc
+	 */
+        ret = xmlSearchNs(out->doc, out, ns->prefix);
+	if ((ret != NULL) && (!xmlStrEqual(ret->href, URI)))
+	    ret = NULL;
+
+	/*
+	 * if the search fails and it's not for the default prefix
+	 * do a search by href
+	 */
+	if ((ret == NULL) && (ns->prefix != NULL))
 	    ret = xmlSearchNsByHref(out->doc, out, URI);
 	}
-    }
 
-    if (ret == NULL) {
+    if (ret == NULL) {	/* if no success and an element node, create the ns */
 	if (out->type == XML_ELEMENT_NODE)
 	    ret = xmlNewNs(out, URI, ns->prefix);
     }
@@ -431,22 +492,26 @@ xsltCopyNamespaceList(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	/*
 	 * Avoid duplicating namespace declrations on the tree
 	 */
-	if ((node != NULL) && (node->ns != NULL) &&
-            (xmlStrEqual(node->ns->href, cur->href)) &&
-            (xmlStrEqual(node->ns->prefix, cur->prefix))) {
-	    cur = cur->next;
-	    continue;
-	}
-	tmp = xmlSearchNs(node->doc, node, cur->prefix);
-	if ((tmp != NULL) && (xmlStrEqual(tmp->href, cur->href))) {
-	    cur = cur->next;
-	    continue;
+	if (node != NULL) {
+	    if ((node->ns != NULL) &&
+        	(xmlStrEqual(node->ns->href, cur->href)) &&
+        	(xmlStrEqual(node->ns->prefix, cur->prefix))) {
+		cur = cur->next;
+		continue;
+	    }
+	    tmp = xmlSearchNs(node->doc, node, cur->prefix);
+	    if ((tmp != NULL) && (xmlStrEqual(tmp->href, cur->href))) {
+		cur = cur->next;
+		continue;
+	    }
 	}
 	
 	if (!xmlStrEqual(cur->href, XSLT_NAMESPACE)) {
 	    /* TODO apply cascading */
 	    URI = (const xmlChar *) xmlHashLookup(ctxt->style->nsAliases,
 		                                  cur->href);
+	    if (URI == UNDEFINED_DEFAULT_NS)
+	        continue;
 	    if (URI != NULL) {
 		q = xmlNewNs(node, URI, cur->prefix);
 	    } else {
@@ -496,6 +561,8 @@ xsltCopyNamespace(xsltTransformContextPtr ctxt, xmlNodePtr node,
     if (!xmlStrEqual(cur->href, XSLT_NAMESPACE)) {
 	URI = (const xmlChar *) xmlHashLookup(ctxt->style->nsAliases,
 					      cur->href);
+	if (URI == UNDEFINED_DEFAULT_NS)
+	    return(NULL);
 	if (URI != NULL) {
 	    ret = xmlNewNs(node, URI, cur->prefix);
 	} else {
